//+------------------------------------------------------------------+
//|                                           EMA_Neuro_GA.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, klot."
#property link      "klot@mail.ru"
//------
#define MAXPOP	   200      //     
#define MAXWEIGHTS	13       //   
//-------
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Aqua
#property indicator_level1 0.0
#property indicator_level2 1.0
#property indicator_level3 -1.0
//---- indicator parameters
//----
extern   int      barn = 150;    //       
extern   double      threshold = 0.7;    //    
//---- indicator buffers
double signal[];
//-------------
extern bool Optimize=false;
static datetime New_Time=0;
//-------------
double ww[MAXWEIGHTS];
//-----
double fitness[MAXPOP];
//-----
double population[MAXPOP][MAXWEIGHTS];  // Population
//+------------------------------------------------------------------+
//|      (gene[])   (int p)           |
//+------------------------------------------------------------------+
void GetGene(int p, double &gene[]) 
{
   for(int j=0; j<MAXWEIGHTS; j++) gene[j]=population[p][j];
   return;
 }
//+------------------------------------------------------------------+
//|                |
//|                                              |
//+------------------------------------------------------------------+
int Solve() {
	int osob = -1;
	double fit=-1;
	// Generate initial population.
	MathSrand(TimeLocal());
   //---    -------------
	for(int i=0; i<MAXPOP; i++) {			
		for (int j=0; j<MAXWEIGHTS; j++) {		
			population[i][j]=(MathRand() % 200)/100.0-1.0;
		}
	}
   //---
	int iterations = 0;	                 // Keep record of the iterations.
	int TotalIterations = 0;
	double Oldfit=-1;
	int Oldosob=-1;
	//+--   ------------------------------------+
	//|    20     +
	//|                               +
	//+-----------------------------------------------------+			
	while ( (iterations<50) ) {
	   //---
      osob=CreateFitnesses(fit);
	   //---        ----
	   if( (osob!=Oldosob) || (fit>Oldfit) ) {
	      //---
	  	   iterations=0;
	      Oldfit=fit;
	      Oldosob=osob;
		}
	   //---
	   CreateNewPopulation(osob);
	   //---
		iterations++;
		TotalIterations++;
	}
   //---
   Print("TotalIterations = ",TotalIterations, "    ",osob," fit = ",fit);
	return (osob);
}
//+------------------------------------------------------------------+
//|                                 |
//|           (double &avgfit)         |
//+------------------------------------------------------------------+
int CreateFitnesses(double &avgfit) {
   //---
   double gen[];
	double max=0;
	int maxFitnessIndex=-1;
   ArrayResize(gen,MAXWEIGHTS);
	//---
	avgfit=0;
	for(int i=0;i<MAXPOP;i++) {
	   //---
	   GetGene(i,gen);
		fitness[i]=TStrategy1(gen,true);
		//---
		if (fitness[i]>max ) {  
		   max=fitness[i];
			maxFitnessIndex=i;
		}
		avgfit += fitness[i];
	}
	avgfit=avgfit/MAXPOP;
	//---
	return(maxFitnessIndex);
}
//+------------------------------------------------------------------+
//|                      |
//|                               |
//+------------------------------------------------------------------+
void CreateNewPopulation(int osob) {
   //---
	int i=0,j=0;
	int crossover=0;
	int mutate=0;
	double xromosoma[];
	ArrayResize(xromosoma,MAXWEIGHTS);
	//---
	GetGene(osob,xromosoma);
	//---
	MathSrand(MathRand());
	//---
	for( i=0; i<MAXPOP; i++ ) {
	   //---
		if ( i==osob ) {	 //     
		 continue; 
		}
		//----        -----
		crossover = (MathRand() % (MAXWEIGHTS-2));
		if ( crossover==0 ) crossover=crossover+1;
		//----  -----
		for( j=crossover; j<MAXWEIGHTS; j++ ) {
			population[i][j]=xromosoma[j];
		}
		//----  (10%)      -----
		mutate=(MathRand() % 100);
		//----
		if( mutate <= 10 ) {
		    for( j=0; j<MAXWEIGHTS; j++ ) {
			   population[i][j]=(MathRand() % 200)/100.0-1.0;
		    }
		}
	}
	//---
	return;
}
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- drawing settings
   SetIndexStyle(0,DRAW_SECTION);
//---- indicator buffers mapping
   SetIndexBuffer(0,signal);
   SetIndexEmptyValue(0,0.0);
//---- indicator short name
   IndicatorShortName("EMA_Neuro_GA("+barn+")");
//---- initialization done
  int i;
  int handle;
  double value;
  //---
  if( !Optimize ) {
   handle=FileOpen("weights2.dat",FILE_BIN);
   if(handle>0) {
      for(i=0; i<MAXWEIGHTS; i++) {
          ww[i]=FileReadDouble(handle,DOUBLE_VALUE);
      }
      FileClose(handle);
   } else {
      //-----
      MathSrand(TimeLocal());
      for(i=0; i<MAXWEIGHTS; i++) ww[i]=(MathRand() % 200)/100.0-1.0;
      //-----
   }
 }
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
      //------
     static int Nosob=-1;
     int nbar=iBarShift(Symbol(),PERIOD_D1,Time[0]);
     if( Optimize==true && New_Time!=iTime(Symbol(),PERIOD_D1,nbar) )  {
         //----
         Nosob=Solve();
         //----
         if( Nosob!=-1 )GetGene(Nosob,ww);
         //----
         writeweights();
         //----
        New_Time=iTime(Symbol(),PERIOD_D1,nbar);
     }
    //---
    double profit=TStrategy1(ww,false);
    //Comment(" = ",profit,"    ",Nosob);
    //-----
 return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double TStrategy1(double wa[],int fOpt=false)
{
   int shift,k=0;
   string name="Signal";
   double in[4];
   int lastbuy=0,lastsell=0,trend=0;
   double profit,profit0;
   double spread=MarketInfo(Symbol(),MODE_SPREAD)*Point;
   double Buy[],Sell[];
   ArrayResize(Buy,barn);
   ArrayResize(Sell,barn);
   ArrayInitialize(Buy,0.0);
   ArrayInitialize(Sell,0.0);
   //--------------T3 ---------------------------------+
   double e1, e2, e3, e4, e5, e6, c1, c2, c3, c4, w1, w2, b2, b3;
   double n,b,t3;
   //----
      n=5;
      b=0.7;
      //-----
      b2=b*b;
      b3=b2*b;
      c1=-b3;
      c2=(3.0*(b2+b3));
      c3=-3.0*(2.0*b2+b+b3);
      c4=(1.0+3.0*b+b3+3*b2);
      //------
      if (n<1) n=1;
      n = 1.0+0.5*(n-1);
      w1 = 2.0/(n+1);
      w2 = 1.0-w1;
   //-------------------------------------------------------------+*/
   //-------------------------------------------------------------
   if(!fOpt) for(shift=barn; shift>=0; shift--) ObjectDelete(name+shift);
   profit=0;
   profit0=0;
   for(shift=barn; shift>=0; shift--) {
      //-----  ----
      in[0]=1000*(iMA(NULL, 0, 4, 0, MODE_EMA, PRICE_CLOSE, shift+1)-
            iMA(NULL, 0, 8, 0, MODE_EMA, PRICE_CLOSE, shift+1)); //LinRegresSlope2(3,shift+1);
      in[1]=1000*(iMA(NULL, 0, 8, 0, MODE_EMA, PRICE_CLOSE, shift+1)-
            iMA(NULL, 0, 16, 0, MODE_EMA, PRICE_CLOSE, shift+1)); //LinRegresSlope2(7,shift+1);
      in[2]=1000*(iMA(NULL, 0, 16, 0, MODE_EMA, PRICE_CLOSE, shift+1)-
            iMA(NULL, 0, 32, 0, MODE_EMA, PRICE_CLOSE, shift+1)); //LinRegresSlope2(14,shift+1);
      in[3]=1000*(iMA(NULL, 0, 32, 0, MODE_EMA, PRICE_CLOSE, shift+1)-
            iMA(NULL, 0, 64, 0, MODE_EMA, PRICE_CLOSE, shift+1)); //LinRegresSlope2(21,shift+1);
      //-----   -------------
      signal[shift] = neuronet1(wa,in);
      //----- T3  --------
      e1 = w1*signal[shift] + w2*e1;
      e2 = w1*e1 + w2*e2;
      e3 = w1*e2 + w2*e3;
      e4 = w1*e3 + w2*e4;
      e5 = w1*e4 + w2*e5;
      e6 = w1*e5 + w2*e6;
      //-----
      t3=c1*e6+c2*e5+c3*e4+c4*e3;
      signal[shift]=t3;///------*/
      //-------------------------------------------------------------+
      //|                                                     |
      //-------------------------------------------------------------+
      if( signal[shift]>signal[shift+1] && signal[shift+2]>signal[shift+1]
          && signal[shift+1]<-threshold && trend!=1 ) {
         Buy[shift]=Close[shift]+spread;
         trend=1;
         lastbuy=shift;
         if(!fOpt){
            ObjectCreate(name+shift,OBJ_ARROW,0,Time[shift],Buy[shift]);
            ObjectSet(name+shift,OBJPROP_ARROWCODE,225);
         }
         k++;
      }
      //------
      if( signal[shift]<signal[shift+1] && signal[shift+2]<signal[shift+1]
          && signal[shift+1]>threshold && trend!=-1 ) { 
         Sell[shift]=Close[shift]-spread;
         trend=-1;
         lastsell=shift;
         if(!fOpt){
            ObjectCreate(name+shift,OBJ_ARROW,0,Time[shift],Sell[shift]);
            ObjectSet(name+shift,OBJPROP_ARROWCODE,226);
         }
         k++;
      }
      //------ -------------
      if( k>1 && ((Buy[shift]!=0 && Sell[lastsell]!=0) ||
                  (Buy[lastbuy]!=0 && Sell[shift]!=0)) )    {
         //-------
         profit+=(Sell[lastsell]-Buy[lastbuy])/Point;
      }
     //------
   }
   //------    -------------
   profit0=0;
   if( lastbuy<lastsell &&  lastbuy>0 ) profit0=(Close[0]-Buy[lastbuy])/Point;
   if( lastsell<lastbuy && lastsell>0 ) profit0=(Sell[lastsell]-Close[0])/Point;
   //-----------------------+
   return(profit+profit0);
}
//+------------------------------------------------------------------+
//| 2-    3-                             |
//|   -                                          |
//|   -                                          |
//|  W[0], W[N+1], W[2*N+2] -              |
//| N -                                                   |
//|      N*2+2+3             |
//+------------------------------------------------------------------+
double neuronet1(double W[],double x[]) 
  {
   int i;
   int N=ArraySize(x);
   if(ArraySize(W)!=N*2+5) {
      Print("!      = ",N*2+5);
      return(0);
   }
   double sum=0.0;
   double Out1=0.0, Out2=0.0;
   //---Neuron 1 ----------------+
   sum=W[0];
   //---
   for(i=1; i<=N; i++) { 
      sum+=W[i]*x[i-1];
   }
   Out1=th(sum);
   //---Neuron 2 ----------------+
   sum=W[N+1];
   //---
   for(i=N+2; i<=2*N+1; i++) { 
      sum+=W[i]*x[i-N-2];
   }
   Out2=th(sum);
   //---Neuron 3 ----------------+
   return(th(Out1*W[2*N+3]+Out2*W[2*N+4]+W[2*N+2]));
  }
  /*
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double LinRegresSlope2(int length, int shift)
{
   int m_pos[2];
   m_pos[0]=shift;
   m_pos[1]=shift+length;
   int n=m_pos[1]-m_pos[0]+1;
//---- calculate price values
   double value=Close[m_pos[0]];
   double a,b,c;
   double sumy=value;
   double sumx=0.0;
   double sumxy=0.0;
   double sumx2=0.0;
   for(int i=1; i<n; i++)
     {
      value=Close[m_pos[0]+i];
      sumy+=value;
      sumxy+=value*i;
      sumx+=i;
      sumx2+=i*i;
     }
   c=sumx2*n-sumx*sumx;
   if(c==0.0) c=0.000001;
   b=(sumxy*n-sumx*sumy)/c;
   a=(sumy-sumx*b)/n;
   //m_value[0]=a;
   //m_value[Extlength]=a+b*n;
   return(th(3000*(a -(a+b*n))/n));
}*/
//+-----------------------------------------------------------------+
//|                                           |
//+-----------------------------------------------------------------+
double th(double x)
{
   double x_=MathExp(x);
   double _x=MathExp(-x);
   //-----
   double a=(x_+_x);
   double b=(x_-_x);
   if( a==0) a=0.00000001;
   return(b/a);
}
//+-----------------------------------------------------------------+
//| writeweights                                                    |
//+-----------------------------------------------------------------+
void writeweights()
{
   int i, handle;
   handle=FileOpen("weights2.dat", FILE_BIN|FILE_WRITE);
   if(handle>0) {
   //---
      for(i=0; i<MAXWEIGHTS; i++) {
         FileWriteDouble(handle, ww[i], DOUBLE_VALUE);
      }
      FileClose(handle);
      return;
   }
   //---
   Print("error open file -",GetLastError());
   return ;
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
  ObjectsDeleteAll();
//----
   return(0);
  }